import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:ocg_app/src/models/response.dart';
import 'package:ocg_app/src/models/token.dart';
import 'package:ocg_app/src/repository/token.dart';

class ApiConfig extends http.BaseClient {
  // static const String ENDPOINT = "13.127.22.122:8080";
  // static const String BASE_URL = "http://$ENDPOINT";
  // static const String BASE_API_URL = "$BASE_URL/api/v1";
  // static const String SOCKET_URL =
  //     "ws://13.127.22.122:8000/connection/websocket?format=protobuf";

  // static const String SUBSCRIBE_URL = "$BASE_API_URL/centrifuge/subscribe";

  final String baseUrl;

  final TokenRepository tokenRepository;
  http.Client _httpClient = new http.Client();

  static Map<String, String> headers = {"Content-Type": 'application/json'};

  ApiConfig(
    this.tokenRepository,
    this.baseUrl,
  );

  Future<Map<String, String>> getAuthenticationHeaders() async {
    AppResponse<AppToken> response = await tokenRepository.getToken();
    if (response.isSuccess) {
      return {"Authorization": "Bearer ${response.data.token}"};
    }
    return {};
  }

  @override
  Future<http.StreamedResponse> send(http.BaseRequest request) async {
    var response = await _httpClient.send(request);

    if (response.statusCode != 401) return response;
    if (!response.headers.containsKey('www-authenticate')) return response;

    return response;
  }

  @override
  Future<http.Response> get(url,
      {Map<String, String> headers, String body}) async {
    if (headers == null) {
      headers = new Map();
    }
    Map<String, String> authenticatedHeaders = await getAuthenticationHeaders();
    headers.addAll(authenticatedHeaders);
    return super
        .get(baseUrl + url, headers: headers)
        .timeout(const Duration(seconds: 10))
        .then((value) => handleResponse(value));
  }

  @override
  Future<http.Response> post(url,
      {Map<String, String> headers, dynamic body, Encoding encoding}) async {
    if (headers == null) {
      headers = new Map();
    }
    Map<String, String> authenticatedHeaders = await getAuthenticationHeaders();
    headers.addAll(authenticatedHeaders);
    return super
        .post(baseUrl + url, headers: headers, body: body, encoding: encoding)
        .timeout(const Duration(seconds: 10))
        .then((value) => handleResponse(value));
  }

  @override
  Future<http.Response> put(
    url, {
    dynamic body,
    Encoding encoding,
    Map<String, String> headers,
  }) async {
    if (headers == null) {
      headers = new Map();
    }
    Map<String, String> authenticatedHeaders = await getAuthenticationHeaders();
    headers.addAll(authenticatedHeaders);
    return super
        .put(baseUrl + url, body: body, encoding: encoding, headers: headers)
        .timeout(const Duration(seconds: 10))
        .then((value) => handleResponse(value));
  }

  @override
  Future<http.Response> delete(
    url, {
    Map<String, String> headers,
  }) async {
    if (headers == null) {
      headers = new Map();
    }
    Map<String, String> authenticatedHeaders = await getAuthenticationHeaders();
    headers.addAll(authenticatedHeaders);
    debugPrint("Url $url");
    return super
        .delete(
          baseUrl + url,
          headers: headers,
        )
        .timeout(const Duration(seconds: 10))
        .then((value) => handleResponse(value));
  }

  http.Response handleResponse(http.Response response) {
    final int statusCode = response.statusCode;
    if (statusCode == 401) {
      throw new Exception("Unauthorized");
    } else {
      try {
        var json = jsonDecode(response.body);
        if (json is Map && json["status"] == "err") {
          throw "${json["msg"] ?? json["err"]}";
        }
      } on TimeoutException {
        debugPrint("Socket Exception");
        throw "It seems like you either have slow or no internet connectivity, hence we cannot refresh at the moment. Please ensure that yout internet in working fine and try again.";
      } on SocketException {
        debugPrint("Socket Exception");
        throw "It seems like you either have slow or no internet connectivity, hence we cannot refresh at the moment. Please ensure that yout internet in working fine and try again.";
      } catch (e) {
        throw e.toString();
      }
    }
    return response;
  }
}
