import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:ocg_app/src/widgets/top_action_widget.dart';
import 'package:ocg_app/src/widgets/video_list_widget.dart';

class VideoScreen extends StatefulWidget {
  final String videoUrl;

  const VideoScreen({Key key, @required this.videoUrl}) : super(key: key);
  @override
  _VideoScreenState createState() => _VideoScreenState();
}

class _VideoScreenState extends State<VideoScreen> {
  bool isFullScreen;

  @override
  void initState() {
    super.initState();
    isFullScreen = true;
    SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeLeft]);
  }

  @override
  Widget build(BuildContext context) {
    return AnnotatedRegion<SystemUiOverlayStyle>(
      value: SystemUiOverlayStyle.light,
      child: Scaffold(
        backgroundColor: Colors.black,
        body: Stack(
          children: [
            VideoListWidget(
              video: widget.videoUrl,
              isFullScreen: isFullScreen,
              onToggleScreen: (val) {
                setState(() {
                  isFullScreen = val;
                });
              },
              onPlayNext: null,
              onPlayPrev: null,
            ),
            Align(
              alignment: Alignment.topLeft,
              child: Padding(
                padding: const EdgeInsets.all(18.0),
                child: TopActionWidget(
                  child: Icon(
                    Icons.clear,
                    color: Colors.white,
                  ),
                  onTap: () {
                    Navigator.of(context).pop();
                  },
                ),
              ),
            )
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
    super.dispose();
  }
}
