import 'dart:math' as math;

import 'package:flutter/material.dart';

class CameraSwitchWidget extends StatefulWidget {
  final IconThemeData themeData;
  CameraSwitchWidget({Key key, this.themeData}) : super(key: key);
  @override
  CameraSwitchWidgetState createState() => CameraSwitchWidgetState();
}

class CameraSwitchWidgetState extends State<CameraSwitchWidget>
    with TickerProviderStateMixin<CameraSwitchWidget> {
  AnimationController _animationController;

  @override
  void initState() {
    super.initState();
    _animationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 400));
  }

  Future<void> switchCamera(bool isFront) async {
    if (isFront) {
      await _animationController.animateTo(0.5);
    } else {
      await _animationController.animateTo(0);
    }
  }

  @override
  Widget build(BuildContext context) {
    IconThemeData themeData =
        widget.themeData ?? IconThemeData(color: Colors.white, size: 24);
    return AnimatedBuilder(
      animation: _animationController,
      builder: (context, child) {
        return Transform.rotate(
          angle: _animationController.value * math.pi,
          child: child,
        );
      },
      child: Icon(
        Icons.camera_alt,
        color: themeData.color,
        size: themeData.size,
      ),
    );
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }
}
